// This is -*- C++ -*-
// $Id: draw.cc,v 1.7 1999/01/07 14:41:41 glaurent Exp $

/* 
 * draw.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

 

#include <string.h>
#include <gdk--/draw.h>
#include <gdk--/pixmap.h>
#include <gdk/gdkprivate.h>
#include <glib.h>


Gdk_Draw::Gdk_Draw() : d1_(0), d2_(0), 
  xo_(0), yo_(0), w_(-1), h_(-1),
  gc_(0)
{ }

Gdk_Draw::Gdk_Draw(Gdk_Drawable& d) : d1_(d), d2_(0),
  xo_(0), yo_(0), w_(-1), h_(-1),
  gc_(0)
{
  // It's really OK for the args to be NULL, e.g. when copying
  //  the drawables from another Gdk_Draw
}

Gdk_Draw::Gdk_Draw(Gdk_Drawable& d1, Gdk_Drawable& d2) : d1_(d1), d2_(d2),
  xo_(0), yo_(0), w_(-1), h_(-1),
  gc_(0)
{
  // ditto.
}

Gdk_Draw::~Gdk_Draw()
{
  // nothing to do, Gdk_GC destructor catches everything.
}

void
Gdk_Draw::set_drawable(Gdk_Drawable& d)
{
  if (!d.connected())
    g_warning("Passed null pointer to Gdk_Draw::set_drawable");
  d1_ = d;

  // When we set change our drawable, clear out any old GC cruft...
  gc_flush_stack();
}

void
Gdk_Draw::set_secondary_drawable(Gdk_Drawable& d)
{
  if (!d.connected())
    g_warning("Passed null pointer to Gdk_Draw::set_drawable");
  d2_ = d;
}

gint
Gdk_Draw::width()
{
  g_return_val_if_fail((w_ > 0) || (d1_.connected()), 0);
  return w_ > 0 ? w_ : ((GdkWindowPrivate*)((GdkDrawable*)drawable()))->width;
}

gint
Gdk_Draw::height()
{
  g_return_val_if_fail((h_ > 0) || (d1_.connected()), 0);
  return h_ > 0 ? h_ : ((GdkWindowPrivate*)((GdkDrawable*)drawable()))->height;
}

void
Gdk_Draw::gc_save()
{
  if (gc_.connected()) {
    gc_stack_.push_back(gc());
    gc_.free();
  }
}

void
Gdk_Draw::gc_clone()
{
  Gdk_GC clone = gc();
  gc_save();
  gc_ = clone;
}

void
Gdk_Draw::gc_restore()
{
  if (gc_.connected()) gc_.free();

  if (!gc_stack_.empty()) {
    gc_ = gc_stack_.back();
    gc_stack_.pop_back();
  }
}

void
Gdk_Draw::gc_flush_stack()
{
  do { gc_restore(); } while (!gc_stack_.empty());
}

void 
Gdk_Draw::gc_push(Gdk_GC& pushme)
{
  gc_save();
  gc_ = pushme;
}

Gdk_GC
Gdk_Draw::gc_pop()
{
  Gdk_GC popme = gc();
  gc_restore();
  return popme;
}


Gdk_Pixmap
Gdk_Draw::copy_rectangle_to_pixmap(gint x, gint y, gint w, gint h)
{
  return copy_rectangle_to_pixmap(gc(),x,y,w,h);
}

Gdk_Pixmap
Gdk_Draw::copy_rectangle_to_pixmap(Gdk_GC& GCx,gint x, gint y, gint w, gint h)
{
  Gdk_Pixmap pix=Gdk_Pixmap(drawable(), w, h, -1);
  pix.draw_pixmap( GCx, drawable(), x, y, 0, 0, w, h);
  return Gdk_Pixmap(pix);
}

void
Gdk_Draw::paste_pixmap(Gdk_GC &GCx, Gdk_Pixmap &pixmap, gint x, gint y )
{ 
  GdkPixmap *pix=(GdkPixmap*)pixmap;
  drawable().draw_pixmap( GCx, pixmap, 0, 0, x, y,
		  ((GdkWindowPrivate*)pix)->width,
		  ((GdkWindowPrivate*)pix)->height);

  if (secondary_drawable().connected())
    secondary_drawable().draw_pixmap( GCx, pixmap, 0, 0, x, y,
		    ((GdkWindowPrivate*)pix)->width,
		    ((GdkWindowPrivate*)pix)->height);

}

void
Gdk_Draw::draw_aligned_string(Gdk_GC& GCx,gint x, gint y, const string& str,
			      gint x_align, gint y_align,
			      gint x_adjust, gint y_adjust)
{

  Gdk_Font font = GCx.get_font();

  gint tw = font.string_width(str);
  if (x_align < 0)
    x -= tw + x_adjust;
  else if (x_align == 0)
    x -= tw/2;
  else
    x += x_adjust;

  gint th = font.height();
  if (y_align == 0)
    y += th/2;
  else if (y_align < 0)
    y -= y_adjust;
  else
    y += th + y_adjust;

  draw_string(GCx,x,y,str);
}

void
Gdk_Draw::draw_text(Gdk_GC& GCx,gint x, gint y, const string& str, 
                    gint text_align)
{

  Gdk_Font font = GCx.get_font();

  gint h = font.height();
  gchar* ss = g_strdup(str.c_str());
  gchar* ss_orig = ss;

  gint w=0;
  if (text_align >= 0) 
    w = font.text_width(str);

  while (*ss) {
    gchar* p = ss;
    while (*p && *p != '\n') ++p;
    bool more_to_do = *p;
    *p = '\0';

    gint fudge = 0;
    if (text_align >= 0) {
      gint lw = font.string_width(ss);
      if (text_align > 0)
	fudge = w - lw;
      else
	fudge = (w - lw)/2;
    }

    draw_string(GCx,x + fudge, y, ss );
    y += h;
    ss = p;
    if (more_to_do) ++ss;
  }

  g_free(ss_orig);
}

#if GDK_VERSION_GT(1,0)
void
Gdk_Draw::draw_aligned_text(Gdk_GC& GCx,gint x, gint y, const string& str,
			    gint text_align,
			    gint x_align, gint y_align,
			    gint x_adjust, gint y_adjust)
{

  Gdk_Font font = GCx.get_font();
  
  gint tw = font.text_width(str);
  if (x_align < 0)
    x -= tw + x_adjust;
  else if (x_align == 0)
    x -= tw/2;
  else
    x += x_adjust;

  y += font.height();
  gint th = font.text_height(str);
  if (y_align == 0)
    y -= th/2;
  else if (y_align < 0)
    y -= th + y_adjust;
  else
    y += y_adjust;

  draw_text(GCx,x,y,str,text_align);
}

#endif




void Gdk_Draw::set_default_clip_rectangle(gint xo, gint yo, gint w, gint h) {
  xo_ = xo; yo_ = yo; w_ = w; h_ = h;
  // Update current GC, if we have one...
  if (gc_.connected()) gc_.set_clip_rectangle(xo_,yo_,w_,h_);
}

void Gdk_Draw::set_default_noclip() {
  xo_ = yo_ = 0;
  w_ = h_ = -1;
  // Again, update current GC...
  if (gc_.connected()) gc_.set_noclip();
}

void Gdk_Draw::set_default_clip_offset_from(Gdk_Draw & source, 
		    gint xoffset,
		    gint yoffset) {
  // A source w/h of -1 will result in a dest w/h depending
  // on buffer size. hrrm.
  set_default_clip_rectangle(source.x_origin() + xoffset,
	       source.y_origin() + yoffset,
	       source.width(),
	       source.height());
}


// By doing this "lazy allocation", we never have to worry about
// creating the GC before we have been realized.
// If we've defined a default clipping rectangle, we apply that rectangle
// to any GC that we create.
Gdk_GC& Gdk_Draw::gc() {
  if (!gc_.connected()) {
    gc_ = Gdk_GC(d1_);
//      gdk_gc_unref(gc_.gdkobj()); // Gdk_GC will keep one reference
    if (w_ > 0 && h_ > 0)
      gc_.set_clip_rectangle(xo_,yo_,w_,h_);
  }
  return gc_;
}


void Gdk_Draw::draw_point(Gdk_GC& myGC,gint x, gint y) 
  { 
   drawable().draw_point(myGC, x, y);
   if (secondary_drawable().connected())
     secondary_drawable().draw_point(myGC, x, y);
  }

void Gdk_Draw::draw_line(Gdk_GC& myGC,gint x0, gint y0, gint x1, gint y1) {
  drawable().draw_line( myGC, x0, y0, x1, y1);
  if (secondary_drawable().connected())
    secondary_drawable().draw_line( myGC, x0, y0, x1, y1);
}

void Gdk_Draw::draw_points(Gdk_GC& myGC,GdkPoint* points, gint npoints) {
  gdk_draw_points(drawable(), myGC, points, npoints);
  if (secondary_drawable().connected())
    gdk_draw_points(secondary_drawable(), myGC, points, npoints);
}

void Gdk_Draw::draw_segments(Gdk_GC &myGC,GdkSegment* segs, gint nsegs) {
  drawable().draw_segments( myGC, segs, nsegs);
  if (secondary_drawable().connected())
    secondary_drawable().draw_segments(myGC, segs, nsegs);
}

void Gdk_Draw::draw_lines(Gdk_GC& myGC,GdkPoint* points, gint npoints) {
  drawable().draw_lines( myGC, points, npoints);
  if (secondary_drawable().connected())
    secondary_drawable().draw_lines( myGC, points, npoints);
}

// draw a rectangle, specifying the (x,y) of the upper left-hand corner
// and the width and height.
void Gdk_Draw::draw_rectangle(Gdk_GC& myGC,bool filled, gint x, gint y, 
                              gint w, gint h) {
  drawable().draw_rectangle( myGC, filled, x, y, w, h);
  if (secondary_drawable().connected())
    secondary_drawable().draw_rectangle( myGC, filled, x, y, w, h);
}

// draw a rectangle, specifying the (x,y) of two of the opposite corners.
void Gdk_Draw::draw_rectangle_coor(Gdk_GC& myGC,bool filled, gint x0, gint y0, 
                         gint x1, gint y1) {
  draw_rectangle(myGC,filled,
   x0 < x1 ? x0 : x1, y0 < y1 ? y0 : y1,
   1+abs(x1-x0), 1+abs(y1-y0));
}

/*
  Annoying X-ism: When specifying the angle for an arc, it has to be given
  as degrees*64.  Therefore an arc that sweeps from 0 to 180 degrees
  should have a1=0, a2=11530 (=180*64).
*/
void Gdk_Draw::draw_arc(Gdk_GC& myGC,bool filled, gint x, gint y, gint w, gint h, 
              gint a1, gint a2) {
  drawable().draw_arc( myGC, filled, x, y, w, h, a1, a2);
  if (secondary_drawable().connected())
    secondary_drawable().draw_arc( myGC, filled, x, y, w, h, a1, a2);
}

// This is usually what I want to do when I call draw_arc() anyway...
void Gdk_Draw::draw_circle(Gdk_GC& myGC,bool filled, gint x, gint y, guint r) {
  draw_arc(myGC,filled,x-r,y-r,2*r,2*r,0,360*64);
}

void Gdk_Draw::draw_polygon(Gdk_GC& myGC,bool filled, GdkPoint* points, gint npoints) {
  drawable().draw_polygon( myGC, filled, points, npoints);
  if (secondary_drawable().connected())
    secondary_drawable().draw_polygon(
       myGC, filled, points, npoints);
}


void Gdk_Draw::draw_string(Gdk_GC& myGC,gint x, gint y, const string& str ) {

  Gdk_Font font = myGC.get_font();

  drawable().draw_string( font, myGC, x, y, str);
  if (secondary_drawable().connected())
    secondary_drawable().draw_string( font, myGC, x, y, str);
}


// $Id: draw.cc,v 1.7 1999/01/07 14:41:41 glaurent Exp $
